/*

 Album Art Widget for Dashboard.
 http://www.liquidx.net/albumartwidget/

 Copyright (c) 2005, Alastair Tse <alastair@liquidx.net>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright notice, this
 list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

 Neither the Alastair Tse nor the names of its contributors may
 be used to endorse or promote products derived from this software without 
 specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
*/


/* Fetch and Parse naver Images */
var naver_base = "http://openapi.naver.com/search";
var naver_params = new Array();
naver_params["query"] = "";
naver_params["target"] = "shop";
naver_params["display"] = "5"
naver_params["start"] = "1";

/* Please Insert Your Naver API Key */
naver_params["key"] = "";


var naver_image_table = '<item>';


/* some global state - inevitable with javascript */

var naver_req = {req: null, on_finish: null, on_error: null};
var naver_req_interval = 1000; // 3 seconds delay
var naver_req_time = 0; // keep track of the last request, and throttle

var naver_throttle_timer = null;
var naver_throttle_args = null;

/* constructs a url for album art fetching */

function naver_make_url(query) {
    var url = naver_base + "?";
    
    naver_params["query"] = query;
    
    for (var i in naver_params) {
        url = url + i + "=" + encodeURIComponent(naver_params[i]) + "&";
    }

    url = url.substring(0,url.length-1);

    return url;
}

/* handler for when the XmlHttpRequest comes back */

function naver_process_request() {
    if (naver_req.req.readyState == 4) { 
        naver_req_time = (new Date).getTime();
        if (naver_req.req.status == 200) {
            if (naver_req.on_finish != null) {
                naver_req.on_finish(naver_req.req);
            }
        }
        else {
            if (naver_req.on_error != null) {
                naver_req.on_error(naver_req.req);
            }
        }
    }
}

/* creates a request - if you execute this faster than the throttle, it will
    either queue your request or silently discard it */

function naver_make_request(query, on_finish, on_error) {
    var url = naver_make_url(query);

    now = (new Date).getTime();

    if (naver_req_time + naver_req_interval > now) {
        naver_throttle_delay(naver_req_interval, query, on_finish, on_error);
        return;
    }
    naver_req_time = now;

    if (window.XMLHttpRequest) {
        naver_req.req = new XMLHttpRequest();
        naver_req.on_finish = on_finish;
        naver_req.on_error = on_error;
        naver_req.req.url = url;
        naver_req.req.onreadystatechange = naver_process_request;
        naver_req.req.open("GET", url, true);
        naver_req.req.send();
    }
}

/* ability to return all the urls returned, and displays them in a DOM
    element called 'images' and a textarea element called 'debug'  */

function naver_get_urls(req) {
    var small_large_urls = new Array();

    // NOTE: reponseXML doesn't exist because output is unparsable    
    if (req.responseText.indexOf(naver_image_table) != -1) {
        // find image matches
        var index = req.responseText.indexOf("<image>");
        while (index != -1) {
            var end_index = req.responseText.indexOf("</image>", index + 1);
            var img_url = req.responseText.substring(index+7, end_index);
            var is_image = img_url.indexOf("http://");
            if (is_image != -1) {
   		        var a_index = req.responseText.indexOf("<link>",index+1);
	            var end_a_index = req.responseText.indexOf("</link>", a_index + 1);
	            var a_url = req.responseText.substring(a_index+6, end_a_index);
                
                var urls = new Array(2);

                urls[0] = img_url;
                urls[1] = img_url;
//                urls[1] = a_url;
                small_large_urls.push(urls);
            }
            index = req.responseText.indexOf("<image>", end_index + 1);
        }
    }
    return small_large_urls;
}

// --------------------------------------------------------------------------
// private functions you don't have to care about
// --------------------------------------------------------------------------

function naver_throttle_delay(delay, query, on_finish, on_error) {

    if (naver_throttle_timer != null) {
        clearInterval(naver_throttle_timer);
        naver_throttle_timer = null;
    }
    naver_throttle_args = new Object
    naver_throttle_args.query = query;
    naver_throttle_args.on_finish = on_finish;
    naver_throttle_args.on_error = on_error;
    naver_throttle_timer = setInterval("naver_throttle_resume();", delay);    
}

function naver_throttle_resume() {
    query = naver_throttle_args.query;
    on_finish = naver_throttle_args.on_finish;
    on_error = naver_throttle_args.on_error;
    
    if (naver_throttle_timer != null) {
        clearInterval(naver_throttle_timer);
        naver_throttle_timer = null;
    }

    naver_make_request(query, on_finish, on_error);
}


// --------------------------------------------------------------------------
// debugging !!!
// --------------------------------------------------------------------------


function test_handler(req) {
    var img_urls = naver_get_urls(req);
    if ((img_urls == null) || (img_urls.length < 1)) {
        return;
    }
    
    var d = document.getElementById("images");
    for (var i = 0; i < img_urls.length; i++) {
        var img = document.createElement("image");
        img.src = img_urls[i][0];
        d.appendChild(img);
        img = document.createElement("image");
        img.src = img_urls[i][1];
        d.appendChild(img);
    }
}


function test_request() {
    var query = document.getElementById("query").value;
    naver_make_request(query, test_handler, alert);
}    


/* To test this, put the code below in an HTML file and play! 
file:///Users/liquidx/Dev/cocoa/AlbumArtWidget/AlbumArt.wdgt/debug.html
<html>
<head>
<script type="text/javascript" src="AlbumArtnaver.js" charset="utf-8" />
</head>
<body>
query: <input type="text" id="query" value="" /><br />
<a href="javascript:test_request();">test</a>
<p><textarea id="debug" cols="80" rows="10"></textarea></p>
<p id="images"></p>
</body>
</html>

*/